@@ -131,6 +131,11 @@ class BrailleGenerator(generator.Generator):
         empty array.  Note that a 'role' attribute in args will
         override the accessible role of the obj.
         """
+
+        if args.get('isProgressBarUpdate') \
+           and not _settingsManager.getSetting('brailleProgressBarUpdates'):
+            return []
+
         result = []
         role = args.get('role', obj.getRole())
         verbosityLevel = _settingsManager.getSetting('brailleVerbosityLevel')
@@ -206,6 +211,11 @@ class BrailleGenerator(generator.Generator):
         specifications) that represent the accelerator for the object,
         or an empty array if no accelerator can be found.
         """
+
+        verbosityLevel = _settingsManager.getSetting('brailleVerbosityLevel')
+        if verbosityLevel == settings.VERBOSITY_LEVEL_BRIEF:
+            return []
+
         result = []
         [mnemonic, shortcut, accelerator] = \
             self._script.utilities.mnemonicShortcutAccelerator(obj)
@@ -314,17 +324,8 @@ class BrailleGenerator(generator.Generator):
         if role not in [pyatspi.ROLE_LIST, pyatspi.ROLE_LIST_BOX]:
             return result
 
-        try:
-            s = obj.querySelection()
-        except NotImplementedError:
-            isFocused = \
-                lambda x: x and x.getState().contains(pyatspi.STATE_FOCUSED)
-            items = pyatspi.utils.findAllDescendants(obj, isFocused)
-        else:
-            items = [s.getSelectedChild(i) for i in range(s.nSelectedChildren)]
-            if not items and obj.childCount:
-                items.append(obj[0])
-
+        items = self._script.utilities.selectedChildren(obj)
+        items = items or [self._script.utilities.focusedChild(obj)]
         if not (items and items[0]):
             return result
 
@@ -334,6 +335,46 @@ class BrailleGenerator(generator.Generator):
 
         return result
 
+    def _generateListBoxItemWidgets(self, obj, **args):
+        widgetRoles = [pyatspi.ROLE_CHECK_BOX,
+                       pyatspi.ROLE_COMBO_BOX,
+                       pyatspi.ROLE_PUSH_BUTTON,
+                       pyatspi.ROLE_RADIO_BUTTON,
+                       pyatspi.ROLE_SLIDER,
+                       pyatspi.ROLE_TOGGLE_BUTTON]
+        isWidget = lambda x: x and x.getRole() in widgetRoles
+        result = []
+        if obj.parent and obj.parent.getRole() == pyatspi.ROLE_LIST_BOX:
+            widgets = pyatspi.findAllDescendants(obj, isWidget)
+            for widget in widgets:
+                result.extend(self.generate(widget, includeContext=False))
+                result.append(braille.Region(" "))
+
+        return result
+
+    def _generateProgressBarIndex(self, obj, **args):
+        if not args.get('isProgressBarUpdate') \
+           or not _settingsManager.getSetting('brailleProgressBarUpdates'):
+            return []
+
+        acc, updateTime, updateValue = self._script.utilities.getMostRecentProgressBarUpdate()
+        if acc != obj:
+            number, count = self._script.utilities.getProgressBarNumberAndCount(obj)
+            return ['%s' % number]
+
+        return []
+
+    def _generateProgressBarValue(self, obj, **args):
+        if args.get('isProgressBarUpdate') \
+           and not _settingsManager.getSetting('brailleProgressBarUpdates'):
+            return []
+
+        percent = self._script.utilities.getValueAsPercent(obj)
+        if percent is not None:
+            return ['%s%%' % percent]
+
+        return []
+
     #####################################################################
     #                                                                   #
     # Unfortunate hacks.                                                #
@@ -389,6 +430,10 @@ class BrailleGenerator(generator.Generator):
         """Returns True or False to indicate whether context should be
         included or not.
         """
+
+        if args.get('isProgressBarUpdate'):
+            return False
+
         # For multiline text areas, we only show the context if we
         # are on the very first line.  Otherwise, we show only the
         # line.
