@@ -96,19 +96,30 @@ class EventManager:
     def _ignore(self, event):
         """Returns True if this event should be ignored."""
 
+        msg = '\nINFO: %s for %s in %s' % (event.type, event.source, event.host_application)
+        debug.println(debug.LEVEL_INFO, msg)
+
         if not self._active:
+            msg = 'INFO: Ignoring because event manager is not active'
+            debug.println(debug.LEVEL_INFO, msg)
             return True
 
         if list(filter(event.type.startswith, self._ignoredEvents)):
+            msg = 'INFO: Ignoring because event type is ignored'
+            debug.println(debug.LEVEL_INFO, msg)
             return True
 
         if event.type.startswith('window'):
+            msg = 'INFO: Not ignoring because event type is never ignored'
+            debug.println(debug.LEVEL_INFO, msg)
             return False
 
         # This should ultimately be changed as there are valid reasons
         # to handle these events at the application level.
         if event.type.startswith('object:children-changed:remove') \
            and event.source != self.registry.getDesktop(0):
+            msg = 'INFO: Ignoring because event type is ignored'
+            debug.println(debug.LEVEL_INFO, msg)
             return True
 
         if event.type.startswith('object:text-changed') and event.type.endswith('system'):
@@ -127,6 +138,7 @@ class EventManager:
             # issue, but until we know for certain....
             #name = event.source.name
             state = event.source.getState()
+            role = event.source.getRole()
         except:
             msg = 'ERROR: %s from potentially-defunct source %s in app %s (%s, %s, %s)' % \
                   (event.type, event.source, event.host_application, event.detail1,
@@ -140,18 +152,26 @@ class EventManager:
             debug.println(debug.LEVEL_INFO, msg)
             return True
 
+        if role in [pyatspi.ROLE_FILLER]:
+            msg = 'INFO: Ignoring because events from this role are ignored'
+            debug.println(debug.LEVEL_INFO, msg)
+            return True
+
         if event.type.startswith('object:state-changed:showing'):
-            try:
-                role = event.source.getRole()
-            except:
-                role = None
-            if role in [pyatspi.ROLE_IMAGE, pyatspi.ROLE_MENU_ITEM, pyatspi.ROLE_PARAGRAPH]:
+            if role not in [pyatspi.ROLE_ALERT,
+                            pyatspi.ROLE_ANIMATION,
+                            pyatspi.ROLE_INFO_BAR,
+                            pyatspi.ROLE_NOTIFICATION,
+                            pyatspi.ROLE_PANEL,
+                            pyatspi.ROLE_STATUS_BAR,
+                            pyatspi.ROLE_TOOL_TIP]:
                 msg = 'INFO: %s for %s in app %s. Who cares?' % \
                       (event.type, event.source, event.host_application)
                 debug.println(debug.LEVEL_INFO, msg)
                 return True
 
-        if event.type.startswith('object:children-changed:add'):
+        if event.type.startswith('object:children-changed:add') \
+           or event.type.startswith('object:active-descendant-changed'):
             if not event.any_data:
                 msg = 'ERROR: %s without child from source %s in app %s' % \
                       (event.type, event.source, event.host_application)
@@ -182,6 +202,8 @@ class EventManager:
                 debug.println(debug.LEVEL_INFO, msg)
                 return True
 
+        msg = 'INFO: Not ignoring due to lack of cause'
+        debug.println(debug.LEVEL_INFO, msg)
         return False
 
     def _addToQueue(self, event, asyncMode):
@@ -663,6 +685,7 @@ class EventManager:
         except:
             msg = 'ERROR: Could not process %s' % event.type
             debug.println(debug.LEVEL_INFO, msg)
+            debug.printException(debug.LEVEL_INFO)
 
     def processKeyboardEvent(self, keyboardEvent):
         """Processes the given keyboard event based on the keybinding from the
