@@ -231,7 +231,7 @@ class Generator:
             debug.println(debug.LEVEL_ALL, "\nPREPARATION TIME: %s" % duration)
             debug.println(
                 debug.LEVEL_ALL,
-                "generate %s for %s %s (args=%s) using '%s'" \
+                "generate %s for %s %s\n(args=%s)\nusing '%s'" \
                 % (self._mode,
                    args['formatType'], 
                    details,
@@ -330,7 +330,7 @@ class Generator:
                 link = None
                 if obj.getRole() == pyatspi.ROLE_LINK:
                     link = obj
-                elif obj.parent.getRole() == pyatspi.ROLE_LINK:
+                elif obj.parent and obj.parent.getRole() == pyatspi.ROLE_LINK:
                     link = obj.parent
                 if link:
                     basename = self._script.utilities.linkBasename(link)
@@ -861,8 +861,7 @@ class Generator:
             return []
 
         substring = self._script.utilities.substring(obj, start, end)
-        if substring and substring.strip() != obj.name \
-           and not self._script.EMBEDDED_OBJECT_CHARACTER in substring:
+        if substring and not self._script.EMBEDDED_OBJECT_CHARACTER in substring:
             return [substring]
 
         return []
@@ -1047,3 +1046,9 @@ class Generator:
 
     def _generatePageSummary(self, obj, **args):
         return []
+
+    def _generateProgressBarIndex(self, obj, **args):
+        return []
+
+    def _generateProgressBarValue(self, obj, **args):
+        return []
