@@ -139,66 +139,51 @@ def setLocusOfFocus(event, obj, notifyScript=True, force=False):
     """
 
     if not force and obj == orca_state.locusOfFocus:
+        msg = "INFO: Setting locusOfFocus to existing locusOfFocus"
+        debug.println(debug.LEVEL_INFO, msg)
         return
 
-    # If this event is not for the currently active script, then just return.
-    #
-    if event and event.source and \
-       event.host_application and orca_state.activeScript:
-        currentApp = orca_state.activeScript.app
-        try:
-            appList = [event.host_application, event.source.getApplication()]
-        except (LookupError, RuntimeError):
-            appList = []
-            debug.println(debug.LEVEL_SEVERE,
-                           "orca.setLocusOfFocus() application Error")
-        if not currentApp in appList:
+    if event and (orca_state.activeScript and not orca_state.activeScript.app):
+        script = _scriptManager.getScript(event.host_application, event.source)
+        _scriptManager.setActiveScript(script, "Setting locusOfFocus")
+
+    oldFocus = orca_state.locusOfFocus
+    try:
+        oldFocus.getRole()
+    except:
+        msg = "INFO: Old locusOfFocus is null or defunct"
+        debug.println(debug.LEVEL_INFO, msg)
+        oldFocus = None
+
+    if not obj:
+        msg = "INFO: New locusOfFocus is null (being cleared)"
+        debug.println(debug.LEVEL_INFO, msg)
+        orca_state.locusOfFocus = None
+        return
+
+    if orca_state.activeScript:
+        if orca_state.activeScript.utilities.isZombie(obj):
+            msg = "ERROR: New locusOfFocus (%s) is zombie. Not updating." % obj
+            debug.println(debug.LEVEL_INFO, msg)
+            return
+        if orca_state.activeScript.utilities.isDead(obj):
+            msg = "ERROR: New locusOfFocus (%s) is dead. Not updating." % obj
+            debug.println(debug.LEVEL_INFO, msg)
             return
 
-    oldLocusOfFocus = orca_state.locusOfFocus
-    try:
-        # Just to see if we have a valid object.
-        oldLocusOfFocus.getRole()
-    except:
-        # Either it's None or it's an invalid remote object.
-        oldLocusOfFocus = None
-
+    msg = "INFO: Changing locusOfFocus from %s to %s" % (oldFocus, obj)
+    debug.println(debug.LEVEL_INFO, msg)
     orca_state.locusOfFocus = obj
-    try:
-        app = orca_state.locusOfFocus.getApplication()
-    except:
-        orca_state.locusOfFocus = None
-        if event:
-            debug.println(debug.LEVEL_FINE,
-                          "LOCUS OF FOCUS: None event='%s'" % event.type)
-        else:
-            debug.println(debug.LEVEL_FINE,
-                          "LOCUS OF FOCUS: None event=None")
-    else:
-        try:
-            appname = "'" + app.name + "'"
-        except:
-            appname = "None"
-        try:
-            name = orca_state.locusOfFocus.name
-            rolename = orca_state.locusOfFocus.getRoleName()
-        except:
-            name = "Error"
-            rolename = "Error"
-        debug.println(debug.LEVEL_FINE,
-                      "LOCUS OF FOCUS: app=%s name='%s' role='%s'" \
-                      % (appname, name, rolename))
 
-        if event:
-            debug.println(debug.LEVEL_FINE,
-                          "                event='%s'" % event.type)
-        else:
-            debug.println(debug.LEVEL_FINE,
-                          "                event=None")
+    if not notifyScript:
+        return
 
-    if notifyScript and orca_state.activeScript:
-        orca_state.activeScript.locusOfFocusChanged(
-            event, oldLocusOfFocus, orca_state.locusOfFocus)
+    if not orca_state.activeScript:
+        msg = "INFO: Cannot notify active script because there isn't one"
+        debug.println(debug.LEVEL_INFO, msg)
+        return
+
+    orca_state.activeScript.locusOfFocusChanged(event, oldFocus, orca_state.locusOfFocus)
 
 ########################################################################
 #                                                                      #
@@ -256,6 +241,8 @@ def _processKeyboardEvent(event):
         return False
 
     if isPressedEvent:
+        if not orca_state.activeWindow:
+            orca_state.activeWindow = script.utilities.activeWindow()
         script.presentationInterrupt()
     script.presentKeyboardEvent(keyboardEvent)
     if keyboardEvent.isModifierKey() and not isOrcaModifier:
