@@ -43,11 +43,12 @@ import orca.speech as speech
 import orca.settings as settings
 import orca.settings_manager as settings_manager
 
-from .speech_generator import SpeechGenerator
 from .braille_generator import BrailleGenerator
 from .formatting import Formatting
-from .structural_navigation import StructuralNavigation
 from .script_utilities import Utilities
+from .spellcheck import SpellCheck
+from .speech_generator import SpeechGenerator
+from .structural_navigation import StructuralNavigation
 
 _settingsManager = settings_manager.getManager()
 
@@ -77,15 +78,6 @@ class Script(default.Script):
         self.dynamicColumnHeaders = {}
         self.dynamicRowHeaders = {}
 
-        # The following variables will be used to try to determine if we've
-        # already handled this misspelt word (see readMisspeltWord() for
-        # more details.
-
-        self.lastTextLength = -1
-        self.lastBadWord = ''
-        self.lastStartOff = -1
-        self.lastEndOff = -1
-
     def getBrailleGenerator(self):
         """Returns the braille generator for this script.
         """
@@ -96,6 +88,11 @@ class Script(default.Script):
         """
         return SpeechGenerator(self)
 
+    def getSpellCheck(self):
+        """Returns the spellcheck for this script."""
+
+        return SpellCheck(self)
+
     def getFormatting(self):
         """Returns the formatting strings for this script."""
         return Formatting(self)
@@ -273,6 +270,8 @@ class Script(default.Script):
         self.skipBlankCellsCheckButton.set_active(value)
         tableGrid.attach(self.skipBlankCellsCheckButton, 0, 3, 1, 1)
 
+        spellcheck = self.spellcheck.getAppPreferencesGUI()
+        grid.attach(spellcheck, 0, len(grid.get_children()), 1, 1)
         grid.show_all()
 
         return grid
@@ -280,7 +279,7 @@ class Script(default.Script):
     def getPreferencesFromGUI(self):
         """Returns a dictionary with the app-specific preferences."""
 
-        return {
+        prefs = {
             'speakCellSpan': self.speakCellSpanCheckButton.get_active(),
             'speakCellHeaders': self.speakCellHeadersCheckButton.get_active(),
             'skipBlankCells': self.skipBlankCellsCheckButton.get_active(),
@@ -288,6 +287,9 @@ class Script(default.Script):
             'speakSpreadsheetCoordinates': self.speakSpreadsheetCoordinatesCheckButton.get_active(),
         }
 
+        prefs.update(self.spellcheck.getPreferencesFromGUI())
+        return prefs
+
     def isStructuralNavigationCommand(self, inputEvent=None):
         """Checks to see if the inputEvent was a structural navigation
         command. This is necessary to prevent double-presentation of
@@ -309,6 +311,14 @@ class Script(default.Script):
 
         return False
 
+    def doWhereAmI(self, inputEvent, basicOnly):
+        """Performs the whereAmI operation."""
+
+        if self.spellcheck.isActive():
+            self.spellcheck.presentErrorDetails(not basicOnly)
+
+        super().doWhereAmI(inputEvent, basicOnly)
+
     def panBrailleLeft(self, inputEvent=None, panAmount=0):
         """In document content, we want to use the panning keys to browse the
         entire document.
@@ -506,114 +516,6 @@ class Script(default.Script):
 
         return True
 
-    def readMisspeltWord(self, event, pane):
-        """Speak/braille the current misspelt word plus its context.
-           The spell check dialog contains a "paragraph" which shows the
-           context for the current spelling mistake. After speaking/brailling
-           the default action for this component, that a selection of the
-           surronding text from that paragraph with the misspelt word is also
-           spoken.
-
-        Arguments:
-        - event: the event.
-        - pane: the option pane in the spell check dialog.
-
-        Returns True if this is the spell check dialog (whether we actually
-        wind up reading the word or not).
-        """
-
-        def isMatch(obj):
-            if not (obj and obj.getRole() == pyatspi.ROLE_PARAGRAPH):
-                return False
-
-            if not obj.getState().contains(pyatspi.STATE_EDITABLE):
-                return False
-
-            try:
-                text = obj.queryText()
-            except:
-                return False
-
-            return text.characterCount > 0
-
-        paragraph = pyatspi.findAllDescendants(pane, isMatch)
-
-        # If there is not exactly one paragraph, this isn't the spellcheck
-        # dialog.
-        #
-        if len(paragraph) != 1:
-            return False
-
-        # If there's not any text displayed in the paragraph, this isn't
-        # the spellcheck dialog.
-        #
-        try:
-            text = paragraph[0].queryText()
-        except:
-            return False
-        else:
-            textLength = text.characterCount
-            if not textLength:
-                return False
-
-        # Determine which word is the misspelt word. This word will have
-        # non-default text attributes associated with it.
-        #
-        startFound = False
-        startOff = 0
-        endOff = textLength
-        for i in range(0, textLength):
-            attributes = text.getAttributes(i)
-            if len(attributes[0]) != 0:
-                if not startFound:
-                    startOff = i
-                    startFound = True
-            else:
-                if startFound:
-                    endOff = i
-                    break
-
-        if not startFound:
-            # If there are no text attributes in this paragraph, this isn't
-            # the spellcheck dialog.
-            #
-            return False
-
-        badWord = self.utilities.substring(paragraph[0], startOff, endOff - 1)
-
-        # Note that we often get two or more of these focus or property-change
-        # events each time there is a new misspelt word. We extract the
-        # length of the line of text, the misspelt word, the start and end
-        # offsets for that word and compare them against the values saved
-        # from the last time this routine was called. If they are the same
-        # then we ignore it.
-        #
-        debug.println(debug.LEVEL_INFO,
-            "StarOffice.readMisspeltWord: type=%s  word=%s(%d,%d)  len=%d" % \
-            (event.type, badWord, startOff, endOff, textLength))
-
-        if (textLength == self.lastTextLength) and \
-           (badWord == self.lastBadWord) and \
-           (startOff == self.lastStartOff) and \
-           (endOff == self.lastEndOff):
-            return True
-
-        # Create a list of all the words found in the misspelt paragraph.
-        #
-        text = self.utilities.substring(paragraph[0], 0, -1)
-        allTokens = text.split()
-        self.speakMisspeltWord(allTokens, badWord)
-
-        # Save misspelt word information for comparison purposes next
-        # time around.
-        #
-        self.lastTextLength = textLength
-        self.lastBadWord = badWord
-        self.lastStartOff = startOff
-        self.lastEndOff = endOff
-
-        return True
-
     def locusOfFocusChanged(self, event, oldLocusOfFocus, newLocusOfFocus):
         """Called when the visual object with focus changes.
 
@@ -635,6 +537,12 @@ class Script(default.Script):
         if self.flatReviewContext:
             self.toggleFlatReviewMode()
 
+        if self.spellcheck.isSuggestionsItem(newLocusOfFocus) \
+           and not self.spellcheck.isSuggestionsItem(oldLocusOfFocus):
+            self.updateBraille(newLocusOfFocus)
+            self.spellcheck.presentSuggestionListItem(includeLabel=True)
+            return
+
         # TODO - JD: Sad hack that wouldn't be needed if LO were fixed.
         # If we are in the slide presentation scroll pane, also announce
         # the current page tab. See bug #538056 for more details.
@@ -694,29 +602,6 @@ class Script(default.Script):
             self.pointOfReference['lastRow'] = row
             self.pointOfReference['lastColumn'] = column
 
-    def onWindowActivated(self, event):
-        """Called whenever a property on an object changes.
-
-        Arguments:
-        - event: the Event
-        """
-
-        self.lastTextLength = -1
-        self.lastBadWord = ''
-        self.lastStartOff = -1
-        self.lastEndOff = -1
-
-        default.Script.onWindowActivated(self, event)
-
-        # Maybe it's the spellcheck dialog. Might as well try and see.
-        # If it is, we want to speak the misspelled word and context
-        # after we've spoken the window name.
-        if event.source \
-           and event.source.getRole() == pyatspi.ROLE_DIALOG \
-           and event.source.childCount \
-           and event.source[0].getRole() == pyatspi.ROLE_OPTION_PANE:
-            self.readMisspeltWord(event, event.source)
-
     def onNameChanged(self, event):
         """Called whenever a property on an object changes.
 
@@ -724,19 +609,8 @@ class Script(default.Script):
         - event: the Event
         """
 
-        # Check to see if if we've had a property-change event for the
-        # accessible name for the option pane in the spell check dialog.
-        # This (hopefully) means that the user has just corrected a
-        # spelling mistake, in which case, speak/braille the current
-        # misspelt word plus its context.
-        #
-        rolesList = [pyatspi.ROLE_OPTION_PANE, \
-                     pyatspi.ROLE_DIALOG, \
-                     pyatspi.ROLE_APPLICATION]
-        if self.utilities.hasMatchingHierarchy(event.source, rolesList) \
-           and self.utilities.isSameObject(
-                event.source.parent, orca_state.activeWindow):
-            self.readMisspeltWord(event, event.source)
+        if self.spellcheck.isCheckWindow(event.source):
+            return
 
         # Impress slide navigation.
         #
@@ -775,6 +649,15 @@ class Script(default.Script):
         if self.utilities.isSameObject(event.any_data, orca_state.locusOfFocus):
             return
 
+        if event.source == self.spellcheck.getSuggestionsList():
+            if event.source.getState().contains(pyatspi.STATE_FOCUSED):
+                orca.setLocusOfFocus(event, event.any_data, False)
+                self.updateBraille(orca_state.locusOfFocus)
+                self.spellcheck.presentSuggestionListItem()
+            else:
+                self.spellcheck.presentErrorDetails()
+            return
+
         default.Script.onActiveDescendantChanged(self, event)
 
     def onChildrenChanged(self, event):
@@ -829,6 +712,9 @@ class Script(default.Script):
         # This callback remains just to handle bugs in applications and toolkits
         # during the remainder of the unstable (3.11) development cycle.
 
+        if self.utilities.isSameObject(orca_state.locusOfFocus, event.source):
+            return
+
         role = event.source.getRole()
 
         # This seems to be something we inherit from Gtk+
@@ -883,16 +769,16 @@ class Script(default.Script):
         if role == pyatspi.ROLE_MENU:
             return
 
-        obj, offset = self.pointOfReference.get("lastCursorPosition", (None, -1))
-        textSelections = self.pointOfReference.get('textSelections', {})
-        start, end = textSelections.get(hash(obj), (0, 0))
-        if start != end:
-            return
-
         if self.utilities._flowsFromOrToSelection(event.source):
             return
 
         if role == pyatspi.ROLE_PARAGRAPH:
+            obj, offset = self.pointOfReference.get("lastCursorPosition", (None, -1))
+            textSelections = self.pointOfReference.get('textSelections', {})
+            start, end = textSelections.get(hash(obj), (0, 0))
+            if start != end:
+                return
+
             keyString, mods = self.utilities.lastKeyAndModifiers()
             if keyString in ["Left", "Right"]:
                 orca.setLocusOfFocus(event, event.source, False)
@@ -900,55 +786,34 @@ class Script(default.Script):
 
         # We should present this in response to active-descendant-changed events
         if event.source.getState().contains(pyatspi.STATE_MANAGES_DESCENDANTS):
-            return
+            if role != pyatspi.ROLE_LIST:
+                return
 
         default.Script.onFocusedChanged(self, event)
 
     def onCaretMoved(self, event):
-        """Called whenever the caret moves.
+        """Callback for object:text-caret-moved accessibility events."""
 
-        Arguments:
-        - event: the Event
-        """
-
-        if self.isStructuralNavigationCommand():
-            return
 
         if event.detail1 == -1:
             return
 
-        if self.utilities.isCellBeingEdited(event.source):
-            orca.setLocusOfFocus(event, event.source.parent, False)
-
-        if not orca_state.locusOfFocus \
-           or self.utilities.isZombie(orca_state.locusOfFocus):
-            orca.setLocusOfFocus(event, event.source)
-            default.Script.onCaretMoved(self, event)
-            return
-
-        if orca_state.locusOfFocus.getRole() == pyatspi.ROLE_TABLE_CELL:
-            default.Script.onCaretMoved(self, event)
-            return
-
-        # The lists and combo boxes in the Formatting toolbar emit
-        # object:active-descendant-changed events which cause us
-        # to set the locusOfFocus to the list item. If the user then
-        # arrows within the text portion, we will not present it due
-        # to the event not being from the locusOfFocus. A similar
-        # issue is present in the Target entry of the Hyperlink dialog
-        # for OOo 3.2.
-        #
-        if event.source.getRole() == pyatspi.ROLE_TEXT \
-           and self.utilities.ancestorWithRole(
-               event.source,
-               [pyatspi.ROLE_TOOL_BAR, pyatspi.ROLE_DIALOG],
-               [pyatspi.ROLE_FRAME]):
-            orca.setLocusOfFocus(event, event.source, False)
-
         if self.utilities._flowsFromOrToSelection(event.source):
+           return
+
+        if self.isStructuralNavigationCommand():
             return
 
-        default.Script.onCaretMoved(self, event)
+        if self.utilities.isSpreadSheetCell(orca_state.locusOfFocus):
+            msg = "SOFFICE: locusOfFocus %s is spreadsheet cell" % orca_state.locusOfFocus
+            debug.println(debug.LEVEL_INFO, msg)
+
+            if not self.utilities.isCellBeingEdited(event.source):
+                msg = "SOFFICE: Event ignored: Source is not cell being edited."
+                debug.println(debug.LEVEL_INFO, msg)
+                return
+
+        super().onCaretMoved(event)
 
     def onCheckedChanged(self, event):
         """Callback for object:state-changed:checked accessibility events."""
@@ -1010,3 +875,21 @@ class Script(default.Script):
             textLine[0] = self.utilities.displayedText(obj)
 
         return textLine
+
+    def onWindowActivated(self, event):
+        """Callback for window:activate accessibility events."""
+
+        super().onWindowActivated(event)
+        if not self.spellcheck.isCheckWindow(event.source):
+            return
+
+        if event.source[0].getRole() == pyatspi.ROLE_DIALOG:
+            orca.setLocusOfFocus(event, event.source[0], False)
+
+        self.spellcheck.presentErrorDetails()
+
+    def onWindowDeactivated(self, event):
+        """Callback for window:deactivate accessibility events."""
+
+        super().onWindowDeactivated(event)
+        self.spellcheck.deactivate()
