@@ -42,8 +42,8 @@ import orca.settings_manager as settings_manager
 import orca.speechserver as speechserver
 import orca.orca_state as orca_state
 import orca.speech as speech
+import orca.structural_navigation as structural_navigation
 
-from .structural_navigation import StructuralNavigation
 from .braille_generator import BrailleGenerator
 from .speech_generator import SpeechGenerator
 from .script_utilities import Utilities
@@ -58,18 +58,15 @@ _settingsManager = settings_manager.getManager()
 
 class Script(default.Script):
 
-    CARET_NAVIGATION_KEYS = ['Left', 'Right', 'Up', 'Down', 'Home', 'End']
-
-    def __init__(self, app, isBrowser=True):
+    def __init__(self, app):
         """Creates a new script for WebKitGtk applications.
 
         Arguments:
         - app: the application to create a script for.
         """
 
-        default.Script.__init__(self, app)
+        super().__init__(app)
         self._loadingDocumentContent = False
-        self._isBrowser = isBrowser
         self._lastCaretContext = None, -1
         self.sayAllOnLoadCheckButton = None
 
@@ -141,38 +138,32 @@ class Script(default.Script):
 
         return SpeechGenerator(self)
 
-    def getStructuralNavigation(self):
-        """Returns the 'structural navigation' class for this script."""
-
-        types = self.getEnabledStructuralNavigationTypes()
-        return StructuralNavigation(self, types, True)
-
     def getEnabledStructuralNavigationTypes(self):
         """Returns a list of the structural navigation object types
         enabled in this script."""
 
-        enabledTypes = [StructuralNavigation.BLOCKQUOTE,
-                        StructuralNavigation.BUTTON,
-                        StructuralNavigation.CHECK_BOX,
-                        StructuralNavigation.CHUNK,
-                        StructuralNavigation.COMBO_BOX,
-                        StructuralNavigation.ENTRY,
-                        StructuralNavigation.FORM_FIELD,
-                        StructuralNavigation.HEADING,
-                        StructuralNavigation.LANDMARK,
-                        StructuralNavigation.LINK,
-                        StructuralNavigation.LIST,
-                        StructuralNavigation.LIST_ITEM,
-                        StructuralNavigation.LIVE_REGION,
-                        StructuralNavigation.PARAGRAPH,
-                        StructuralNavigation.RADIO_BUTTON,
-                        StructuralNavigation.SEPARATOR,
-                        StructuralNavigation.TABLE,
-                        StructuralNavigation.TABLE_CELL,
-                        StructuralNavigation.UNVISITED_LINK,
-                        StructuralNavigation.VISITED_LINK]
-
-        return enabledTypes
+        return [structural_navigation.StructuralNavigation.BLOCKQUOTE,
+                structural_navigation.StructuralNavigation.BUTTON,
+                structural_navigation.StructuralNavigation.CHECK_BOX,
+                structural_navigation.StructuralNavigation.CHUNK,
+                structural_navigation.StructuralNavigation.CLICKABLE,
+                structural_navigation.StructuralNavigation.COMBO_BOX,
+                structural_navigation.StructuralNavigation.ENTRY,
+                structural_navigation.StructuralNavigation.FORM_FIELD,
+                structural_navigation.StructuralNavigation.HEADING,
+                structural_navigation.StructuralNavigation.IMAGE,
+                structural_navigation.StructuralNavigation.LANDMARK,
+                structural_navigation.StructuralNavigation.LINK,
+                structural_navigation.StructuralNavigation.LIST,
+                structural_navigation.StructuralNavigation.LIST_ITEM,
+                structural_navigation.StructuralNavigation.LIVE_REGION,
+                structural_navigation.StructuralNavigation.PARAGRAPH,
+                structural_navigation.StructuralNavigation.RADIO_BUTTON,
+                structural_navigation.StructuralNavigation.SEPARATOR,
+                structural_navigation.StructuralNavigation.TABLE,
+                structural_navigation.StructuralNavigation.TABLE_CELL,
+                structural_navigation.StructuralNavigation.UNVISITED_LINK,
+                structural_navigation.StructuralNavigation.VISITED_LINK]
 
     def getUtilities(self):
         """Returns the utilites for this script."""
@@ -180,16 +171,14 @@ class Script(default.Script):
         return Utilities(self)
 
     def onCaretMoved(self, event):
-        """Called whenever the caret moves.
-
-        Arguments:
-        - event: the Event
-        """
+        """Callback for object:text-caret-moved accessibility events."""
 
         if self._inSayAll:
             return
 
-        self._lastCaretContext = event.source, event.detail1
+        if not self.utilities.isWebKitGtk(event.source):
+            super().onCaretMoved(event)
+            return
 
         lastKey, mods = self.utilities.lastKeyAndModifiers()
         if lastKey in ['Tab', 'ISO_Left_Tab']:
@@ -202,33 +191,28 @@ class Script(default.Script):
             self.updateBraille(event.source)
             return
 
-        orca.setLocusOfFocus(event, event.source, False)
-
-        default.Script.onCaretMoved(self, event)
+        self.utilities.setCaretContext(event.source, event.detail1)
+        super().onCaretMoved(event)
 
     def onDocumentReload(self, event):
         """Callback for document:reload accessibility events."""
 
-        docRoles = [pyatspi.ROLE_DOCUMENT_FRAME, pyatspi.ROLE_DOCUMENT_WEB]
-        if event.source.getRole() in docRoles:
+        if self.utilities.treatAsBrowser(event.source):
             self._loadingDocumentContent = True
 
     def onDocumentLoadComplete(self, event):
         """Callback for document:load-complete accessibility events."""
 
-        docRoles = [pyatspi.ROLE_DOCUMENT_FRAME, pyatspi.ROLE_DOCUMENT_WEB]
-        if not event.source.getRole() in docRoles:
+        if not self.utilities.treatAsBrowser(event.source):
             return
 
         self._loadingDocumentContent = False
-        if not self._isBrowser:
-            return
 
         # TODO: We need to see what happens in Epiphany on pages where focus
         # is grabbed rather than set the caret at the start. But for simple
         # content in both Yelp and Epiphany this is alright for now.
-        obj, offset = self.setCaretAtStart(event.source)
-        orca.setLocusOfFocus(event, obj, False)
+        obj, offset = self.utilities.setCaretAtStart(event.source)
+        self.utilities.setCaretContext(obj, offset)
 
         self.updateBraille(obj)
         if _settingsManager.getSetting('sayAllOnLoad') \
@@ -238,45 +222,35 @@ class Script(default.Script):
     def onDocumentLoadStopped(self, event):
         """Callback for document:load-stopped accessibility events."""
 
-        docRoles = [pyatspi.ROLE_DOCUMENT_FRAME, pyatspi.ROLE_DOCUMENT_WEB]
-        if event.source.getRole() in docRoles:
+        if self.utilities.treatAsBrowser(event.source):
             self._loadingDocumentContent = False
 
     def onFocusedChanged(self, event):
         """Callback for object:state-changed:focused accessibility events."""
 
-        if self._inSayAll:
+        if self._inSayAll or not event.detail1:
+            return
+
+        if not self.utilities.isWebKitGtk(event.source):
+            super().onFocusedChanged(event)
+            return
+
+        contextObj, offset = self.utilities.getCaretContext()
+        if event.source == contextObj:
             return
 
         obj = event.source
         role = obj.getRole()
-        self._lastCaretContext = obj, -1
-
-        if role == pyatspi.ROLE_LIST_ITEM and obj.childCount:
-            return
-
-        widgetRoles = [pyatspi.ROLE_MENU,
-                       pyatspi.ROLE_MENU_ITEM,
-                       pyatspi.ROLE_LIST_ITEM,
-                       pyatspi.ROLE_RADIO_BUTTON]
-        if role in widgetRoles:
-            default.Script.onFocusedChanged(self, event)
-            return
-
         textRoles = [pyatspi.ROLE_HEADING,
                      pyatspi.ROLE_PANEL,
                      pyatspi.ROLE_PARAGRAPH,
                      pyatspi.ROLE_SECTION,
                      pyatspi.ROLE_TABLE_CELL]
-        if role in textRoles:
+        if role in textRoles \
+           or (role == pyatspi.ROLE_LIST_ITEM and obj.childCount):
             return
 
-        if not (role == pyatspi.ROLE_LINK and obj.childCount):
-            lastKey, mods = self.utilities.lastKeyAndModifiers()
-            if lastKey in self.CARET_NAVIGATION_KEYS:
-                return
-
-        default.Script.onFocusedChanged(self, event)
+        super().onFocusedChanged(event)
 
     def onBusyChanged(self, event):
         """Callback for object:state-changed:busy accessibility events."""
@@ -288,8 +262,7 @@ class Script(default.Script):
         except:
             return
 
-        docRoles = [pyatspi.ROLE_DOCUMENT_FRAME, pyatspi.ROLE_DOCUMENT_WEB]
-        if role not in docRoles or not self._isBrowser:
+        if not self.utilities.treatAsBrowser(obj):
             return
 
         if event.detail1:
@@ -439,39 +412,6 @@ class Script(default.Script):
 
         return True
 
-    def setCaretAtStart(self, obj):
-        """Attempts to set the caret at the specified offset in obj. Because
-        this is not always possible, this method will attempt to locate the
-        first place inside of obj in which the caret can be positioned.
-
-        Arguments:
-        - obj: the accessible object in which the caret should be placed.
-
-        Returns the object and offset in which we were able to set the caret.
-        Otherwise, None if we could not find a text object, and -1 if we were
-        not able to set the caret.
-        """
-
-        def implementsText(obj):
-            if obj.getRole() == pyatspi.ROLE_LIST:
-                return False
-            return 'Text' in utils.listInterfaces(obj)
-
-        child = obj
-        if not implementsText(obj):
-            child = utils.findDescendant(obj, implementsText)
-            if not child:
-                return None, -1
-
-        index = -1
-        text = child.queryText()
-        for i in range(text.characterCount):
-            if text.setCaretOffset(i):
-                index = i
-                break
-
-        return child, index
-
     def panBrailleLeft(self, inputEvent=None, panAmount=0):
         """In document content, we want to use the panning keys to browse the
         entire document.
@@ -566,6 +506,7 @@ class Script(default.Script):
         spoken and acss is an ACSS instance for speaking the text.
         """
 
+        self._sayAllIsInterrupted = False
         self._inSayAll = False
         if not obj:
             return
@@ -620,6 +561,7 @@ class Script(default.Script):
         text = obj.queryText()
 
         if progressType == speechserver.SayAllContext.INTERRUPTED:
+            self._sayAllIsInterrupted = True
             if isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent):
                 lastKey = orca_state.lastInputEvent.event_string
                 if lastKey == "Down" and self._fastForwardSayAll(context):
@@ -629,7 +571,8 @@ class Script(default.Script):
 
             self._inSayAll = False
             self._sayAllContexts = []
-            text.setCaretOffset(offset)
+            if not self._lastCommandWasStructNav:
+                text.setCaretOffset(offset)
             return
 
         # SayAllContext.COMPLETED doesn't necessarily mean done with SayAll;
@@ -648,20 +591,10 @@ class Script(default.Script):
 
         text.setCaretOffset(offset)
 
-    def getTextLineAtCaret(self, obj, offset=None):
-        """Gets the line of text where the caret is.
+    def getTextLineAtCaret(self, obj, offset=None, startOffset=None, endOffset=None):
+        """To-be-removed. Returns the string, caretOffset, startOffset."""
 
-        Argument:
-        - obj: an Accessible object that implements the AccessibleText
-          interface
-        - offset: an optional caret offset to use. (Not used here at the
-          moment, but needed in the Gecko script)
-
-        Returns the [string, caretOffset, startOffset] for the line of text
-        where the caret is.
-        """
-
-        textLine = default.Script.getTextLineAtCaret(self, obj, offset)
+        textLine = super().getTextLineAtCaret(obj, offset, startOffset, endOffset)
         string = textLine[0]
         if string and string.find(self.EMBEDDED_OBJECT_CHARACTER) == -1 \
            and obj.getState().contains(pyatspi.STATE_FOCUSED):
@@ -677,12 +610,11 @@ class Script(default.Script):
 
         return textLine
 
-    def updateBraille(self, obj, extraRegion=None):
+    def updateBraille(self, obj, **args):
         """Updates the braille display to show the given object.
 
         Arguments:
         - obj: the Accessible
-        - extra: extra Region to add to the end
         """
 
         if not _settingsManager.getSetting('enableBraille') \
@@ -696,7 +628,7 @@ class Script(default.Script):
         if not self.utilities.isWebKitGtk(obj) \
            or (not self.utilities.isInlineContainer(obj) \
                and not self.utilities.isTextListItem(obj)):
-            default.Script.updateBraille(self, obj, extraRegion)
+            default.Script.updateBraille(self, obj, **args)
             return
 
         brailleLine = self.getNewBrailleLine(clearBraille=True, addLine=True)
@@ -712,6 +644,7 @@ class Script(default.Script):
             self.addBrailleRegionsToLine(regions, brailleLine)
             self.setBrailleFocus(fRegion)
 
+        extraRegion = args.get('extraRegion')
         if extraRegion:
             self.addBrailleRegionToLine(extraRegion, brailleLine)
 
