@@ -183,19 +183,42 @@ class Script(default.Script):
             orca.setLocusOfFocus(event, obj)
             return
 
+    def onSelectedChanged(self, event):
+        """Callback for object:state-changed:selected accessibility events."""
+
+        if self.utilities.isEntryCompletionPopupItem(event.source):
+            if event.detail1:
+                orca.setLocusOfFocus(event, event.source)
+                return
+            if orca_state.locusOfFocus == event.source:
+                orca.setLocusOfFocus(event, None)
+                return
+
+        super().onSelectedChanged(event)
+
+    def onSelectionChanged(self, event):
+        """Callback for object:selection-changed accessibility events."""
+
+        if event.source.getRole() == pyatspi.ROLE_COMBO_BOX \
+           and not event.source.getState().contains(pyatspi.STATE_FOCUSED):
+            return
+
+        default.Script.onSelectionChanged(self, event)
+
     def onShowingChanged(self, event):
         """Callback for object:state-changed:showing accessibility events."""
 
-        obj = event.source
-        if not self.utilities._isNonModalPopOver(obj):
-            default.Script.onShowingChanged(self, event)
+        if not event.detail1:
+            super().onShowingChanged(event)
             return
 
-        if event.detail1:
-            speech.speak(self.speechGenerator.generateSpeech(obj))
-            labels = self.utilities.unrelatedLabels(obj)
-            msg = ' '.join(map(self.utilities.displayedText, labels))
-            self.presentMessage(msg)
+        obj = event.source
+        if self.utilities._isNonModalPopOver(obj) \
+           or obj.getRole() in [pyatspi.ROLE_ALERT, pyatspi.ROLE_INFO_BAR]:
+            self.presentObject(event.source)
+            return
+
+        super().onShowingChanged(event)
 
     def onTextSelectionChanged(self, event):
         """Callback for object:text-selection-changed accessibility events."""
