@@ -499,6 +499,8 @@ class Script(default.Script):
         """Creates a generator that can be used to iterate document content."""
 
         if not self.utilities.inDocumentContent():
+            msg = "WEB: textLines called for non-document content %s" % obj
+            debug.println(debug.LEVEL_INFO, msg)
             super().textLines(obj, offset)
             return
 
@@ -549,6 +551,10 @@ class Script(default.Script):
         self._sayAllContents = []
         self._sayAllContexts = []
 
+        msg = "WEB: textLines complete. Verifying SayAll status"
+        debug.println(debug.LEVEL_INFO, msg)
+        self.inSayAll()
+
     def presentFindResults(self, obj, offset):
         """Updates the context and presents the find results if appropriate."""
 
@@ -585,13 +591,14 @@ class Script(default.Script):
         """
 
         if not self.utilities.inDocumentContent():
+            msg = "WEB: SayAll called for non-document content %s" % obj
+            debug.println(debug.LEVEL_INFO, msg)
             return super().sayAll(inputEvent, obj, offset)
 
-        else:
-            obj = obj or orca_state.locusOfFocus
-            speech.sayAll(self.textLines(obj, offset),
-                          self.__sayAllProgressCallback)
-
+        obj = obj or orca_state.locusOfFocus
+        msg = "WEB: SayAll called for document content %s" % obj
+        debug.println(debug.LEVEL_INFO, msg)
+        speech.sayAll(self.textLines(obj, offset), self.__sayAllProgressCallback)
         return True
 
     def _rewindSayAll(self, context, minCharCount=10):
@@ -740,7 +747,7 @@ class Script(default.Script):
         self.displayContents(contents)
         self.speakContents(contents)
 
-    def updateBraille(self, obj, extraRegion=None):
+    def updateBraille(self, obj, **args):
         """Updates the braille display to show the given object."""
 
         if not _settingsManager.getSetting('enableBraille') \
@@ -750,7 +757,7 @@ class Script(default.Script):
 
         if not (self._lastCommandWasCaretNav or self._lastCommandWasStructNav) \
            or self._inFocusMode or not self.utilities.inDocumentContent():
-            super().updateBraille(obj, extraRegion)
+            super().updateBraille(obj, **args)
             return
 
         obj, offset = self.utilities.getCaretContext(documentFrame=None)
@@ -928,10 +935,12 @@ class Script(default.Script):
         self._inMouseOverObject = False
         self._lastMouseOverObject = None
 
-    def enableStickyFocusMode(self, inputEvent):
+    def enableStickyFocusMode(self, inputEvent, forceMessage=False):
+        if not self._focusModeIsSticky or forceMessage:
+            self.presentMessage(messages.MODE_FOCUS_IS_STICKY)
+
         self._inFocusMode = True
         self._focusModeIsSticky = True
-        self.presentMessage(messages.MODE_FOCUS_IS_STICKY)
 
     def togglePresentationMode(self, inputEvent):
         if self._inFocusMode:
@@ -975,6 +984,11 @@ class Script(default.Script):
         if self.utilities.inFindToolbar(oldFocus):
             newFocus, caretOffset = self.utilities.getCaretContext()
 
+        if newFocus.getRole() == pyatspi.ROLE_UNKNOWN:
+            msg = "WEB: Event source has bogus role. Likely browser bug."
+            debug.println(debug.LEVEL_INFO, msg)
+            newFocus, offset = self.utilities.findFirstCaretContext(newFocus, 0)
+
         text = self.utilities.queryNonEmptyText(newFocus)
         if text and (0 <= text.caretOffset < text.characterCount):
             caretOffset = text.caretOffset
@@ -984,6 +998,11 @@ class Script(default.Script):
         speech.speak(self.speechGenerator.generateSpeech(newFocus, priorObj=oldFocus))
         self._saveFocusedObjectInfo(newFocus)
 
+        if self.utilities.inTopLevelWebApp(newFocus):
+            announce = not self.utilities.inDocumentContent(oldFocus)
+            self.enableStickyFocusMode(None, announce)
+            return True
+
         if not self._focusModeIsSticky \
            and self.useFocusMode(newFocus) != self._inFocusMode:
             self.togglePresentationMode(None)
@@ -1049,10 +1068,24 @@ class Script(default.Script):
             return True
 
         self.utilities.clearCachedObjects()
-        if self.useFocusMode(orca_state.locusOfFocus) != self._inFocusMode:
-            self.togglePresentationMode(None)
 
         obj, offset = self.utilities.getCaretContext()
+
+        try:
+            sourceIsBusy = event.souce.getState().contains(pyatspi.STATE_BUSY)
+        except:
+            sourceIsBusy = False
+
+        if not sourceIsBusy and self.utilities.isTopLevelWebApp(event.source):
+            msg = "WEB: Setting locusOfFocus to %s with sticky focus mode" % obj
+            debug.println(debug.LEVEL_INFO, msg)
+            orca.setLocusOfFocus(event, obj)
+            self.enableStickyFocusMode(None, True)
+            return True
+
+        if self.useFocusMode(obj) != self._inFocusMode:
+            self.togglePresentationMode(None)
+
         if not obj:
             msg = "WEB: Could not get caret context"
             debug.println(debug.LEVEL_INFO, msg)
@@ -1243,6 +1276,11 @@ class Script(default.Script):
     def onChildrenChanged(self, event):
         """Callback for object:children-changed accessibility events."""
 
+        if self.utilities.eventIsChromeNoise(event):
+            msg = "WEB: Ignoring event believed to be chrome noise"
+            debug.println(debug.LEVEL_INFO, msg)
+            return True
+
         document = self.utilities.getDocumentForObject(event.source)
         if document:
             msg = "WEB: Clearing structural navigation cache for %s" % document
@@ -1288,6 +1326,13 @@ class Script(default.Script):
         obj, offset = self.utilities.getCaretContext()
         contextDocument = self.utilities.getDocumentForObject(obj)
         if obj and document == contextDocument and self.utilities.isZombie(obj):
+            msg = "WEB: Context obj %s is Zombie." % obj
+            debug.println(debug.LEVEL_INFO, msg)
+            if self._inFocusMode:
+                msg = "WEB: Not looking for replicant due to focus mode."
+                debug.println(debug.LEVEL_INFO, msg)
+                return False
+
             replicant = self.utilities.findReplicant(event.source, obj)
             if replicant:
                 # Refrain from actually touching the replicant by grabbing
@@ -1355,11 +1400,18 @@ class Script(default.Script):
             debug.println(debug.LEVEL_INFO, msg)
             return True
 
-        if not self.utilities.inDocumentContent(event.source):
-            msg = "WEB: Event source is not in document content"
+        document = self.utilities.getDocumentForObject(event.source)
+        if not document:
+            msg = "WEB: Could not get document for event source"
             debug.println(debug.LEVEL_INFO, msg)
             return False
 
+        if self.utilities.isTopLevelWebApp(document):
+            msg = "WEB: Event handled: Setting locusOfFocus to event source"
+            debug.println(debug.LEVEL_INFO, msg)
+            orca.setLocusOfFocus(event, event.source)
+            return True
+
         state = event.source.getState()
         if state.contains(pyatspi.STATE_EDITABLE):
             msg = "WEB: Event source is editable"
@@ -1383,32 +1435,48 @@ class Script(default.Script):
             debug.println(debug.LEVEL_INFO, msg)
             return True
 
-        if role in [pyatspi.ROLE_DOCUMENT_FRAME, pyatspi.ROLE_DOCUMENT_WEB]:
-            obj, offset = self.utilities.getCaretContext(event.source)
-            if obj and self.utilities.isZombie(obj):
-                msg = "WEB: Clearing context - obj is zombie"
-                debug.println(debug.LEVEL_INFO, msg)
-                self.utilities.clearCaretContext()
-                obj, offset = self.utilities.getCaretContext(event.source)
-
-            if obj:
-                wasFocused = obj.getState().contains(pyatspi.STATE_FOCUSED)
-                obj.clearCache()
-                isFocused = obj.getState().contains(pyatspi.STATE_FOCUSED)
-                if wasFocused == isFocused \
-                   and not (obj.getRole() == pyatspi.ROLE_LINK and not isFocused):
-                    msg = "WEB: Event handled: Setting locusOfFocus to context"
-                    debug.println(debug.LEVEL_INFO, msg)
-                    orca.setLocusOfFocus(event, obj)
-                    return True
-
         if not state.contains(pyatspi.STATE_FOCUSABLE) \
            and not state.contains(pyatspi.STATE_FOCUSED):
             msg = "WEB: Event ignored: Source is not focusable or focused"
             debug.println(debug.LEVEL_INFO, msg)
             return True
 
-        return False
+        if not role in [pyatspi.ROLE_DOCUMENT_FRAME, pyatspi.ROLE_DOCUMENT_WEB]:
+            msg = "WEB: Deferring to other scripts for handling non-document source"
+            debug.println(debug.LEVEL_INFO, msg)
+            return False
+
+        obj, offset = self.utilities.getCaretContext(event.source)
+        if obj and self.utilities.isZombie(obj):
+            msg = "WEB: Clearing context - obj is zombie"
+            debug.println(debug.LEVEL_INFO, msg)
+            self.utilities.clearCaretContext()
+            obj, offset = self.utilities.getCaretContext(event.source)
+
+        if not obj:
+            msg = "WEB: Unable to get non-null, non-zombie context object"
+            debug.println(debug.LEVEL_INFO, msg)
+            return False
+
+        wasFocused = obj.getState().contains(pyatspi.STATE_FOCUSED)
+        obj.clearCache()
+        isFocused = obj.getState().contains(pyatspi.STATE_FOCUSED)
+        if wasFocused != isFocused:
+            msg = "WEB: Focused state of %s changed to %s" % (obj, isFocused)
+            debug.println(debug.LEVEL_INFO, msg)
+            return False
+
+        if not (self.utilities.isLink(obj) and not isFocused):
+            cause = "Context is not a non-focused link"
+        elif self.utilities.isChildOfCurrentFragment(obj):
+            cause = "Context is child of current fragment"
+        else:
+            return False
+
+        msg = "WEB: Event handled: Setting locusOfFocus to %s (%s)" % (obj, cause)
+        debug.println(debug.LEVEL_INFO, msg)
+        orca.setLocusOfFocus(event, obj)
+        return True
 
     def onMouseButton(self, event):
         """Callback for mouse:button accessibility events."""
@@ -1421,8 +1489,8 @@ class Script(default.Script):
     def onNameChanged(self, event):
         """Callback for object:property-change:accessible-name events."""
 
-        if self.utilities.eventIsStatusBarNoise(event):
-            msg = "WEB: Ignoring event believed to be status bar noise"
+        if self.utilities.eventIsChromeNoise(event):
+            msg = "WEB: Ignoring event believed to be chrome noise"
             debug.println(debug.LEVEL_INFO, msg)
             return True
 
@@ -1499,8 +1567,8 @@ class Script(default.Script):
             debug.println(debug.LEVEL_INFO, msg)
             return True
 
-        if self.utilities.eventIsStatusBarNoise(event):
-            msg = "WEB: Ignoring event believed to be status bar noise"
+        if self.utilities.eventIsChromeNoise(event):
+            msg = "WEB: Ignoring event believed to be chrome noise"
             debug.println(debug.LEVEL_INFO, msg)
             return True
 
@@ -1550,8 +1618,8 @@ class Script(default.Script):
             debug.println(debug.LEVEL_INFO, msg)
             return True
 
-        if self.utilities.eventIsStatusBarNoise(event):
-            msg = "WEB: Ignoring event believed to be status bar noise"
+        if self.utilities.eventIsChromeNoise(event):
+            msg = "WEB: Ignoring event believed to be chrome noise"
             debug.println(debug.LEVEL_INFO, msg)
             return True
 
@@ -1594,11 +1662,17 @@ class Script(default.Script):
             return True
 
         state = event.source.getState()
-        if not state.contains(pyatspi.STATE_EDITABLE) \
-           and event.source != orca_state.locusOfFocus:
-            msg = "WEB: Done processing non-editable, non-locusOfFocus source"
-            debug.println(debug.LEVEL_INFO, msg)
-            return True
+        if not state.contains(pyatspi.STATE_EDITABLE):
+            if event.source != orca_state.locusOfFocus:
+                msg = "WEB: Done processing non-editable, non-locusOfFocus source"
+                debug.println(debug.LEVEL_INFO, msg)
+                return True
+
+            if self.utilities.isClickableElement(event.source):
+                msg = "WEB: Event handled: Re-setting locusOfFocus to changed clickable"
+                debug.println(debug.LEVEL_INFO, msg)
+                orca.setLocusOfFocus(None, event.source, force=True)
+                return True
 
         return False
 
@@ -1661,7 +1735,10 @@ class Script(default.Script):
     def onWindowActivated(self, event):
         """Callback for window:activate accessibility events."""
 
-        return False
+        msg = "WEB: Calling default onWindowActivated"
+        debug.println(debug.LEVEL_INFO, msg)
+        super().onWindowActivated(event)
+        return True
 
     def onWindowDeactivated(self, event):
         """Callback for window:deactivate accessibility events."""
