@@ -48,15 +48,18 @@ class Utilities(script_utilities.Utilities):
         self._currentAttrs = {}
         self._caretContexts = {}
         self._inDocumentContent = {}
+        self._inTopLevelWebApp = {}
         self._isTextBlockElement = {}
         self._isGridDescendant = {}
         self._isLayoutOnly = {}
         self._isMath = {}
         self._mathNestingLevel = {}
         self._isOffScreenLabel = {}
+        self._hasExplicitName = {}
         self._hasNoSize = {}
         self._hasLongDesc = {}
         self._hasUselessCanvasDescendant = {}
+        self._id = {}
         self._isClickableElement = {}
         self._isAnchor = {}
         self._isLandmark = {}
@@ -77,6 +80,8 @@ class Utilities(script_utilities.Utilities):
         self._currentWordContents = None
         self._currentCharacterContents = None
 
+        self._validChildRoles = {pyatspi.ROLE_LIST: [pyatspi.ROLE_LIST_ITEM]}
+
     def _cleanupContexts(self):
         toRemove = []
         for key, [obj, offset] in self._caretContexts.items():
@@ -89,15 +94,18 @@ class Utilities(script_utilities.Utilities):
     def clearCachedObjects(self):
         debug.println(debug.LEVEL_INFO, "WEB: cleaning up cached objects")
         self._inDocumentContent = {}
+        self._inTopLevelWebApp = {}
         self._isTextBlockElement = {}
         self._isGridDescendant = {}
         self._isLayoutOnly = {}
         self._isMath = {}
         self._mathNestingLevel = {}
         self._isOffScreenLabel = {}
+        self._hasExplicitName = {}
         self._hasNoSize = {}
         self._hasLongDesc = {}
         self._hasUselessCanvasDescendant = {}
+        self._id = {}
         self._isClickableElement = {}
         self._isAnchor = {}
         self._isLandmark = {}
@@ -175,24 +183,28 @@ class Utilities(script_utilities.Utilities):
         if not targets:
             return []
 
-        roles = [pyatspi.ROLE_DOCUMENT_FRAME, pyatspi.ROLE_DOCUMENT_WEB]
-        isDocument = lambda x: x and x.getRole() in roles
-        return list(filter(isDocument, targets))
+        return list(filter(self.isDocument, targets))
 
     def documentFrame(self, obj=None):
-        isShowing = lambda x: x and x.getState().contains(pyatspi.STATE_SHOWING)
-
-        try:
-            windows = [child for child in self._script.app]
-        except:
-            msg = "WEB: Exception getting children for app %s" % self._script.app
+        app = self._script.app
+        if orca_state.activeWindow in app:
+            window = orca_state.activeWindow
+        else:
+            msg = "WARNING: %s is not in %s" % (orca_state.activeWindow, app)
             debug.println(debug.LEVEL_INFO, msg)
-            windows = []
+            window = self.activeWindow(app)
+            try:
+                self._script.app = window.getApplication()
+                msg = "WEB: updating script's app to %s" % self._script.app
+                debug.println(debug.LEVEL_INFO, msg)
+            except:
+                msg = "ERROR: Exception getting app for %s" % window
+                debug.println(debug.LEVEL_INFO, msg)
+            else:
+                orca_state.activeWindow = window
 
-        if orca_state.activeWindow in windows:
-            windows = [orca_state.activeWindow]
-
-        for window in windows:
+        if window:
+            isShowing = lambda x: x and x.getState().contains(pyatspi.STATE_SHOWING)
             documents = self._getDocumentsEmbeddedBy(window)
             documents = list(filter(isShowing, documents))
             if len(documents) == 1:
@@ -333,6 +345,9 @@ class Utilities(script_utilities.Utilities):
             else:
                 break
 
+        if lastChild and self.doNotDescendForCaret(lastChild):
+            lastChild = lastChild.parent
+
         return lastChild
 
     def getRoleDescription(self, obj):
@@ -349,6 +364,19 @@ class Utilities(script_utilities.Utilities):
         self._roleDescription[hash(obj)] = rv
         return rv
 
+    def _getID(self, obj):
+        if hash(obj) in self._id:
+            return self._id.get(hash(obj))
+
+        try:
+            attrs = dict([attr.split(':', 1) for attr in obj.getAttributes()])
+        except:
+            return None
+
+        rv = attrs.get('id')
+        self._id[hash(obj)] = rv
+        return rv
+
     def _getTag(self, obj):
         rv = self._tag.get(hash(obj))
         if rv is not None:
@@ -655,7 +683,8 @@ class Utilities(script_utilities.Utilities):
                 rv = None
             if rv and (self.isHidden(obj) or self.isOffScreenLabel(obj)):
                 rv = None
-            if rv and role == pyatspi.ROLE_LINK and self.hasUselessCanvasDescendant(obj):
+            if rv and role == pyatspi.ROLE_LINK \
+               and (self.hasExplicitName(obj) or self.hasUselessCanvasDescendant(obj)):
                 rv = None
 
         self._text[hash(obj)] = rv
@@ -1273,6 +1302,39 @@ class Utilities(script_utilities.Utilities):
 
         return contents
 
+    def inTopLevelWebApp(self, obj=None):
+        if not obj:
+            obj = orca_state.locusOfFocus
+
+        rv = self._inTopLevelWebApp.get(hash(obj))
+        if rv is not None:
+            return rv
+
+        document = self.getDocumentForObject(obj)
+        if not document and self.isDocument(obj):
+            document = obj
+
+        rv = self.isTopLevelWebApp(document)
+        self._inTopLevelWebApp[hash(obj)] = rv
+        return rv
+
+    def isTopLevelWebApp(self, obj):
+        try:
+            role = obj.getRole()
+        except:
+            msg = "WEB: Exception getting role for %s" % obj
+            debug.println(debug.LEVEL_INFO, msg)
+            return False
+
+        if role == pyatspi.ROLE_EMBEDDED and not self.getDocumentForObject(obj.parent):
+            uri = self.documentFrameURI()
+            rv = bool(uri and uri.startswith("http"))
+            msg = "WEB: %s is top-level web application: %s (URI: %s)" % (obj, rv, uri)
+            debug.println(debug.LEVEL_INFO, msg)
+            return rv
+
+        return False
+
     def isFocusModeWidget(self, obj):
         try:
             role = obj.getRole()
@@ -1287,6 +1349,7 @@ class Utilities(script_utilities.Utilities):
             return True
 
         focusModeRoles = [pyatspi.ROLE_COMBO_BOX,
+                          pyatspi.ROLE_EMBEDDED,
                           pyatspi.ROLE_ENTRY,
                           pyatspi.ROLE_LIST_BOX,
                           pyatspi.ROLE_LIST_ITEM,
@@ -1338,10 +1401,8 @@ class Utilities(script_utilities.Utilities):
                              pyatspi.ROLE_FOOTER,
                              pyatspi.ROLE_FORM,
                              pyatspi.ROLE_HEADING,
-                             pyatspi.ROLE_LABEL,
                              pyatspi.ROLE_LIST,
                              pyatspi.ROLE_LIST_ITEM,
-                             pyatspi.ROLE_PANEL,
                              pyatspi.ROLE_PARAGRAPH,
                              pyatspi.ROLE_ROW_HEADER,
                              pyatspi.ROLE_SECTION,
@@ -1369,6 +1430,9 @@ class Utilities(script_utilities.Utilities):
         return rv
 
     def treatAsDiv(self, obj):
+        if not (obj and self.inDocumentContent(obj)):
+            return False
+
         rv = self._treatAsDiv.get(hash(obj))
         if rv is not None:
             return rv
@@ -1382,8 +1446,18 @@ class Utilities(script_utilities.Utilities):
             return False
 
         rv = False
-        if role == pyatspi.ROLE_LIST:
-            rv = not (childCount and obj[0].getRole() == pyatspi.ROLE_LIST_ITEM)
+
+        validRoles = self._validChildRoles.get(role)
+        if validRoles:
+            if not childCount:
+                rv = True
+            else:
+                rv = bool([x for x in obj if x and x.getRole() not in validRoles])
+
+        if not rv:
+            validRoles = self._validChildRoles.get(obj.parent)
+            if validRoles:
+                rv = bool([x for x in obj.parent if x and x.getRole() not in validRoles])
 
         self._treatAsDiv[hash(obj)] = rv
         return rv
@@ -1701,6 +1775,7 @@ class Utilities(script_utilities.Utilities):
             if (self.isTextBlockElement(obj) and not string.strip()) \
                or self.isAnchor(obj) \
                or (self.hasNoSize(obj) and not string.strip()) \
+               or self.isHidden(obj) \
                or self.isOffScreenLabel(obj) \
                or self.isUselessImage(obj) \
                or self.isLabellingContents(x, contents):
@@ -1956,7 +2031,7 @@ class Utilities(script_utilities.Utilities):
                 attrs = dict([attr.split(':', 1) for attr in obj.getAttributes()])
             except:
                 attrs = {}
-            rv = attrs.get('xml-roles') in settings.ariaLandmarks
+            rv = attrs.get('xml-roles') in self.getLandmarkTypes()
 
         self._isLandmark[hash(obj)] = rv
         return rv
@@ -2097,6 +2172,23 @@ class Utilities(script_utilities.Utilities):
         self._isParentOfNullChild[hash(obj)] = rv
         return rv
 
+    def hasExplicitName(self, obj):
+        if not (obj and self.inDocumentContent(obj)):
+            return False
+
+        rv = self._hasExplicitName.get(hash(obj))
+        if rv is not None:
+            return rv
+
+        try:
+            attrs = dict([attr.split(':', 1) for attr in obj.getAttributes()])
+        except:
+            attrs = {}
+
+        rv = attrs.get('explicit-name') == 'true'
+        self._hasExplicitName[hash(obj)] = rv
+        return rv
+
     def hasLongDesc(self, obj):
         if not (obj and self.inDocumentContent(obj)):
             return False
@@ -2141,9 +2233,8 @@ class Utilities(script_utilities.Utilities):
         if self._script.inSayAll():
             return False
 
-        if not self.inDocumentContent():
+        if not self.inDocumentContent() or self.inTopLevelWebApp():
             return False
-
         try:
             role = obj.getRole()
         except:
@@ -2201,19 +2292,22 @@ class Utilities(script_utilities.Utilities):
 
         return False
 
-    def eventIsStatusBarNoise(self, event):
+    def eventIsChromeNoise(self, event):
         if self.inDocumentContent(event.source):
             return False
 
+        try:
+            role = event.source.getRole()
+        except:
+            msg = "WEB: Exception getting role for %s" % event.source
+            debug.println(debug.LEVEL_INFO, msg)
+            return False
+
         eType = event.type
         if eType.startswith("object:text-") or eType.endswith("accessible-name"):
-            try:
-                role = event.source.getRole()
-            except:
-                msg = "WEB: Exception getting role for %s" % event.source
-                debug.println(debug.LEVEL_INFO, msg)
-            else:
-                return role == pyatspi.ROLE_STATUS_BAR
+            return role in [pyatspi.ROLE_STATUS_BAR, pyatspi.ROLE_LABEL]
+        if eType.startswith("object:children-changed"):
+            return True
 
         return False
 
@@ -2311,8 +2405,21 @@ class Utilities(script_utilities.Utilities):
         if linkURI == docURI:
             return True
 
+        sourceID = self._getID(event.source)
+        if sourceID:
+            parseResult = urllib.parse.urlparse(docURI)
+            return parseResult.fragment == sourceID
+
         return False
 
+    def isChildOfCurrentFragment(self, obj):
+        parseResult = urllib.parse.urlparse(self.documentFrameURI())
+        if not parseResult.fragment:
+            return False
+
+        isSameFragment = lambda x: self._getID(x) == parseResult.fragment
+        return pyatspi.findAncestor(obj, isSameFragment) is not None
+
     @staticmethod
     def getHyperlinkRange(obj):
         try:
@@ -2373,25 +2480,13 @@ class Utilities(script_utilities.Utilities):
 
     def hasNoSize(self, obj):
         if not (obj and self.inDocumentContent(obj)):
-            return False
+            return super().hasNoSize(obj)
 
         rv = self._hasNoSize.get(hash(obj))
         if rv is not None:
             return rv
 
-        try:
-            extents = obj.queryComponent().getExtents(0)
-        except:
-            msg = "WEB: Exception getting extents for %s" % obj
-            debug.println(debug.LEVEL_INFO, msg)
-            rv = True
-        else:
-            rv = not (extents.width and extents.height)
-            if rv:
-                msg = "WEB: %s has no size %s" % (obj, extents)
-                debug.println(debug.LEVEL_INFO, msg)
-
-        self._hasNoSize[hash(obj)] = rv
+        self._hasNoSize[hash(obj)] = super().hasNoSize(obj)
         return rv
 
     def doNotDescendForCaret(self, obj):
@@ -2411,7 +2506,8 @@ class Utilities(script_utilities.Utilities):
             return True
 
         role = obj.getRole()
-        if role == pyatspi.ROLE_LINK and self.hasUselessCanvasDescendant(obj):
+        if role == pyatspi.ROLE_LINK \
+           and (self.hasExplicitName(obj) or self.hasUselessCanvasDescendant(obj)):
             return True
 
         if self.isTextBlockElement(obj):
@@ -2506,7 +2602,7 @@ class Utilities(script_utilities.Utilities):
                        pyatspi.ROLE_INTERNAL_FRAME,
                        pyatspi.ROLE_TABLE,
                        pyatspi.ROLE_TABLE_ROW]
-        if role in lookInChild and obj.childCount:
+        if role in lookInChild and obj.childCount and not self.treatAsDiv(obj):
             msg = "WEB: First caret context for %s, %i will look in child %s" % (obj, offset, obj[0])
             debug.println(debug.LEVEL_INFO, msg)
             return self.findFirstCaretContext(obj[0], 0)
