@@ -46,7 +46,10 @@ userCustomizableSettings = [
     "enableEchoByWord",
     "enableEchoBySentence",
     "enableKeyEcho",
-    "enablePrintableKeys",
+    "enableAlphabeticKeys",
+    "enableNumericKeys",
+    "enablePunctuationKeys",
+    "enableSpace",
     "enableModifierKeys",
     "enableFunctionKeys",
     "enableActionKeys",
@@ -71,11 +74,14 @@ userCustomizableSettings = [
     "keyboardLayout",
     "speakBlankLines",
     "speakMultiCaseStringsAsWords",
+    "speakNumbersAsDigits",
     "enabledSpokenTextAttributes",
     "enabledBrailledTextAttributes",
     "textAttributesBrailleIndicator",
-    "enableProgressBarUpdates",
     "profile",
+    "speakProgressBarUpdates",
+    "brailleProgressBarUpdates",
+    "beepProgressBarUpdates",
     "progressBarUpdateInterval",
     "progressBarVerbosity",
     "enableContractedBraille",
@@ -90,8 +96,6 @@ userCustomizableSettings = [
     "largeObjectTextLength",
     "structuralNavigationEnabled",
     "wrappedStructuralNavigation",
-    "brailleRequiredStateString",
-    "speechRequiredStateString",
     "chatMessageVerbosity",
     "chatSpeakRoomName",
     "chatAnnounceBuddyTyping",
@@ -99,8 +103,8 @@ userCustomizableSettings = [
     "enableFlashMessages",
     "brailleFlashTime",
     "flashIsPersistent",
-    "flashVerbosityLevel",
-    "messageVerbosityLevel",
+    "flashIsDetailed",
+    "messagesAreDetailed",
     "presentDateFormat",
     "presentTimeFormat",
     "activeProfile",
@@ -114,6 +118,8 @@ userCustomizableSettings = [
     "structNavTriggersFocusMode",
     "caretNavTriggersFocusMode",
     "layoutMode",
+    "rewindAndFastForwardInSayAll",
+    "structNavInSayAll"
 ]
 
 GENERAL_KEYBOARD_LAYOUT_DESKTOP = 1
@@ -202,13 +208,14 @@ speakCellSpan                = True
 speakCellHeaders             = True
 speakSpreadsheetCoordinates  = True
 speakMultiCaseStringsAsWords = False
+speakNumbersAsDigits         = False
 useColorNames                = True
 usePronunciationDictionary   = True
 sayAllStyle                  = SAYALL_STYLE_SENTENCE
 capitalizationStyle          = CAPITALIZATION_STYLE_NONE
 verbalizePunctuationStyle    = PUNCTUATION_STYLE_MOST
 speechVerbosityLevel         = VERBOSITY_LEVEL_VERBOSE
-messageVerbosityLevel        = VERBOSITY_LEVEL_VERBOSE
+messagesAreDetailed          = True
 enablePauseBreaks            = True
 
 # Braille
@@ -219,6 +226,7 @@ enableBrailleContext           = True
 enableFlashMessages            = True
 brailleFlashTime               = 5000
 flashIsPersistent              = False
+flashIsDetailed                = True
 enableContractedBraille        = False
 brailleContractionTable        = ''
 disableBrailleEOL              = False
@@ -227,7 +235,6 @@ brailleSelectorIndicator       = BRAILLE_UNDERLINE_BOTH
 brailleLinkIndicator           = BRAILLE_UNDERLINE_BOTH
 textAttributesBrailleIndicator = BRAILLE_UNDERLINE_NONE
 brailleVerbosityLevel          = VERBOSITY_LEVEL_VERBOSE
-flashVerbosityLevel            = VERBOSITY_LEVEL_VERBOSE
 brailleAlignmentStyle          = BRAILLE_ALIGN_BY_EDGE
 brailleAlignmentMargin         = 3
 brailleMaximumJump             = 8
@@ -237,7 +244,10 @@ keyboardLayout               = GENERAL_KEYBOARD_LAYOUT_DESKTOP
 orcaModifierKeys             = DESKTOP_MODIFIER_KEYS
 doubleClickTimeout           = 0.5
 enableKeyEcho                = True
-enablePrintableKeys          = True
+enableAlphabeticKeys         = True
+enableNumericKeys            = True
+enablePunctuationKeys        = True
+enableSpace                  = True
 enableModifierKeys           = True
 enableFunctionKeys           = True
 enableActionKeys             = True
@@ -254,7 +264,9 @@ mouseDwellDelay            = 0
 mouseDwellMaxDrift         = 3
 
 # Progressbars
-enableProgressBarUpdates   = True
+speakProgressBarUpdates    = True
+brailleProgressBarUpdates  = False
+beepProgressBarUpdates     = False
 progressBarUpdateInterval  = 10
 progressBarVerbosity       = PROGRESS_BAR_APPLICATION
 
@@ -264,17 +276,6 @@ skipBlankCells              = False
 largeObjectTextLength       = 75
 wrappedStructuralNavigation = True
 inferLiveRegions            = True
-ariaLandmarks = [
-    "application",
-    "banner",
-    "complementary",
-    "contentinfo",
-    "form",
-    "main",
-    "navigation",
-    "region",
-    "search",
-]
 
 # Chat
 chatMessageVerbosity       = CHAT_SPEAK_ALL
@@ -350,11 +351,5 @@ caretNavTriggersFocusMode = False
 
 layoutMode = True
 
-# NOTE: The following are experimental and may be changed or removed at
-# any time
 rewindAndFastForwardInSayAll = False
 structNavInSayAll = False
-
-# NOTE: The following are experimental and may be changed or removed at
-# any time
-disableDeadAccessibleFilter = False
