@@ -510,6 +510,14 @@ class SpeechGenerator(generator.Generator):
     #                                                                   #
     #####################################################################
 
+    def generateLinkInfo(self, obj, **args):
+        result = self._generateLinkInfo(obj, **args)
+        result.extend(self._generatePause(obj, **args))
+        result.append(self._generateSiteDescription(obj, **args))
+        result.extend(self._generatePause(obj, **args))
+        result.append(self._generateFileSize(obj, **args))
+        return result
+
     def _generateLinkInfo(self, obj, **args):
         """Returns an array of strings (and possibly voice and audio
         specifications) that represent the protocol of the URI of
@@ -543,6 +551,14 @@ class SpeechGenerator(generator.Generator):
             else:
                 linkOutput = messages.LINK_WITH_PROTOCOL % link_uri_info[0]
                 text = self._script.utilities.displayedText(obj)
+                try:
+                    isVisited = obj.getState().contains(pyatspi.STATE_VISITED)
+                except:
+                    isVisited = False
+                if not isVisited:
+                    linkOutput = messages.LINK_WITH_PROTOCOL % link_uri_info[0]
+                else:
+                    linkOutput = messages.LINK_WITH_PROTOCOL_VISITED % link_uri_info[0]
                 if not text:
                     # If there's no text for the link, expose part of the
                     # URI to the user.
@@ -1452,17 +1468,8 @@ class SpeechGenerator(generator.Generator):
         if role not in [pyatspi.ROLE_LIST, pyatspi.ROLE_LIST_BOX]:
             return result
 
-        try:
-            s = obj.querySelection()
-        except NotImplementedError:
-            isFocused = \
-                lambda x: x and x.getState().contains(pyatspi.STATE_FOCUSED)
-            items = pyatspi.utils.findAllDescendants(obj, isFocused)
-        else:
-            items = [s.getSelectedChild(i) for i in range(s.nSelectedChildren)]
-            if not items and obj.childCount:
-                items.append(obj[0])
-
+        items = self._script.utilities.selectedChildren(obj)
+        items = items or [self._script.utilities.focusedChild(obj)]
         if not (items and items[0]):
             return result
 
@@ -1717,6 +1724,33 @@ class SpeechGenerator(generator.Generator):
         result.extend(acss)
         return result
 
+    def _generateProgressBarIndex(self, obj, **args):
+        if not args.get('isProgressBarUpdate') \
+           or not _settingsManager.getSetting('speakProgressBarUpdates'):
+            return []
+
+        result = []
+        acc, updateTime, updateValue = self._script.utilities.getMostRecentProgressBarUpdate()
+        if acc != obj:
+            number, count = self._script.utilities.getProgressBarNumberAndCount(obj)
+            result = [messages.PROGRESS_BAR_NUMBER % (number)]
+            result.extend(self.voice(SYSTEM))
+
+        return result
+
+    def _generateProgressBarValue(self, obj, **args):
+        if args.get('isProgressBarUpdate') \
+           and not _settingsManager.getSetting('speakProgressBarUpdates'):
+            return []
+
+        result = []
+        percent = self._script.utilities.getValueAsPercent(obj)
+        if percent is not None:
+            result.append(messages.percentage(percent))
+            result.extend(self.voice(SYSTEM))
+
+        return result
+
     def _generateDefaultButton(self, obj, **args):
         """Returns an array of strings (and possibly voice and audio
         specifications) that represent the default button in a dialog.
@@ -1788,6 +1822,22 @@ class SpeechGenerator(generator.Generator):
             result.append(dialogs)
         return result
 
+    def _generateListBoxItemWidgets(self, obj, **args):
+        widgetRoles = [pyatspi.ROLE_CHECK_BOX,
+                       pyatspi.ROLE_COMBO_BOX,
+                       pyatspi.ROLE_PUSH_BUTTON,
+                       pyatspi.ROLE_RADIO_BUTTON,
+                       pyatspi.ROLE_SLIDER,
+                       pyatspi.ROLE_TOGGLE_BUTTON]
+        isWidget = lambda x: x and x.getRole() in widgetRoles
+        result = []
+        if obj.parent and obj.parent.getRole() == pyatspi.ROLE_LIST_BOX:
+            widgets = pyatspi.findAllDescendants(obj, isWidget)
+            for widget in widgets:
+                result.append(self.generate(widget, includeContext=False))
+
+        return result
+
     #####################################################################
     #                                                                   #
     # Keyboard shortcut information                                     #
